part of katana_module;

/// Module adapter for setting up market place purchasing.
///
/// [MarketPlaceAdapter] can switch the data
/// when the module is used by passing it to [UIMaterialApp].
@immutable
abstract class MarketPlaceAdapter<TProduct, TPurchaseOption, TAccountDocument,
    TPaymentMethodCollection, TPurchaseCollection> extends AdapterModule {
  const MarketPlaceAdapter();

  /// Class for managing market place process.
  ///
  /// Initialize by first executing [initialize()].
  Future<void> initialize();

  /// Retrieve account documentation as it is monitored.
  TAccountDocument? watchAccountDocument(WidgetRef ref);

  /// Retrieve while monitoring the PaymentMethod collection.
  TPaymentMethodCollection? watchPaymentMethodCollection(WidgetRef ref);

  /// Retrieve while monitoring the Purchase collection.
  TPurchaseCollection? watchPurchaseCollection(WidgetRef ref);

  /// Seller account registration.
  Future<void> registrationSeller();

  /// View the dashboard for sellers & buyers.
  Future<void> showDashboard();

  /// Set up your payment method.
  Future<void> setupPaymentMethod();

  /// Set the default payment method to the one with [paymentMethodId].
  Future<void> setDefaultPaymentMethod(String paymentMethodId);

  /// Deletes the PaymentMethod with the ID of [paymentMethodId].
  Future<void> deletePaymentMethod(String paymentMethodId);

  /// Purchase the [product].
  ///
  /// Basically, after making a purchase, you must confirm the purchase by clicking [confirm].
  ///
  /// The billing is then executed by executing [capture].
  Future<void> purchase(TProduct product);

  /// Update the information of the purchase in [orderId].
  Future<void> refresh(String orderId);

  /// Finalize the purchase related to [orderId].
  ///
  /// Options can be set at the time of finalization by specifying [options].
  ///
  /// It is necessary that [purchase] is executed beforehand.
  Future<void> confirm(
    String orderId, {
    TPurchaseOption? options,
  });

  /// Issue invoices for purchases associated with [orderId].
  ///
  /// Options can be set at the time of finalization by specifying [options].
  ///
  /// It is necessary that [purchase] and [confirm] is executed beforehand.
  Future<void> capture(
    String orderId, {
    TPurchaseOption? options,
  });

  /// Cancels the purchase for [orderId].
  Future<void> cancel(String orderId);

  /// Runs after authentication has taken place.
  ///
  /// It is also called after registration or login has been completed.
  @override
  @mustCallSuper
  Future<void> onAfterAuth(BuildContext context) async {
    await super.onAfterAuth(context);
    await initialize();
  }
}
